@@ -51,6 +51,8 @@ package com.openexchange.realtime.hazelcast.cleanup;
 
 import java.util.Collection;
 import java.util.HashSet;
+import java.util.Iterator;
+import java.util.Map.Entry;
 import java.util.Set;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
@@ -61,13 +63,16 @@ import com.openexchange.management.ManagementAware;
 import com.openexchange.management.ManagementObject;
 import com.openexchange.realtime.cleanup.GlobalRealtimeCleanup;
 import com.openexchange.realtime.cleanup.LocalRealtimeCleanup;
+import com.openexchange.realtime.directory.Resource;
 import com.openexchange.realtime.exception.RealtimeExceptionCodes;
 import com.openexchange.realtime.hazelcast.channel.HazelcastAccess;
 import com.openexchange.realtime.hazelcast.directory.HazelcastResourceDirectory;
 import com.openexchange.realtime.hazelcast.management.GlobalRealtimeCleanupMBean;
 import com.openexchange.realtime.hazelcast.management.GlobalRealtimeCleanupManagement;
 import com.openexchange.realtime.hazelcast.osgi.Services;
+import com.openexchange.realtime.hazelcast.serialization.cleanup.PortableCleanupDispatcher;
 import com.openexchange.realtime.packet.ID;
+import com.openexchange.realtime.util.IDMap;
 
 /**
  * {@link GlobalRealtimeCleanupImpl}
@@ -93,14 +98,6 @@ public class GlobalRealtimeCleanupImpl implements GlobalRealtimeCleanup, Managem
     @Override
     public void cleanForId(ID id) {
         LOG.debug("Starting global realtime cleanup for ID: {}", id);
-        try {
-            Collection<ID> removeFromResourceDirectory = removeFromResourceDirectory(id);
-            if(removeFromResourceDirectory.isEmpty()) {
-                LOG.debug("Unable to remove {} from ResourceDirectory.", id);
-            }
-        } catch (OXException oxe) {
-            LOG.error("Unable to remove {} from ResourceDirectory.", id, oxe);
-        }
 
         // Do the local cleanup via a simple service call
         LocalRealtimeCleanup localRealtimeCleanup = Services.optService(LocalRealtimeCleanup.class);
@@ -112,6 +109,16 @@ public class GlobalRealtimeCleanupImpl implements GlobalRealtimeCleanup, Managem
             localRealtimeCleanup.cleanForId(id);
         }
 
+        //Remove from directory after the RealtimeJanitors ran so we don't end up with an unreachable directory entry
+        try {
+            Collection<ID> removeFromResourceDirectory = removeFromResourceDirectory(id);
+            if(removeFromResourceDirectory.isEmpty()) {
+                LOG.debug("Unable to remove {} from ResourceDirectory.", id);
+            }
+        } catch (OXException oxe) {
+            LOG.error("Unable to remove {} from ResourceDirectory.", id, oxe);
+        }
+
         //Remote cleanup via distributed MultiTask to remaining members of the cluster
         HazelcastInstance hazelcastInstance;
         try {
@@ -122,7 +129,7 @@ public class GlobalRealtimeCleanupImpl implements GlobalRealtimeCleanup, Managem
                 LOG.warn("Couldn't remove local member from cluster members.");
             }
             if(!clusterMembers.isEmpty()) {
-                hazelcastInstance.getExecutorService("default").submitToMembers(new CleanupDispatcher(id), clusterMembers);
+                hazelcastInstance.getExecutorService("default").submitToMembers(new PortableCleanupDispatcher(id), clusterMembers);
             } else {
                 LOG.debug("No other cluster members besides the local member. No further clean up necessary.");
             }
@@ -131,6 +138,27 @@ public class GlobalRealtimeCleanupImpl implements GlobalRealtimeCleanup, Managem
         }
     }
 
+    @Override
+    public void cleanForId(ID id, long timestamp) {
+        try {
+            IDMap<Resource> idMap = hazelcastResourceDirectory.get(id);
+            Set<ID> validIDs = new HashSet<ID>();
+            Iterator<Entry<ID, Resource>> mapIter = idMap.entrySet().iterator();
+            while (mapIter.hasNext()) {
+                Entry<ID, Resource> next = mapIter.next();
+                if (next.getValue().getTimestamp().getTime() <= timestamp) {
+                    validIDs.add(next.getKey());
+                }
+            }
+            hazelcastResourceDirectory.remove(validIDs);
+            for (ID idToClean : validIDs) {
+                doCleanupForId(idToClean);
+            }
+        } catch (OXException oxe) {
+            LOG.error("Failed to clean for ID {}", id, oxe);
+        }
+    }
+
     @Override
     public Collection<ID> removeFromResourceDirectory(ID id) throws OXException {
         return hazelcastResourceDirectory.remove(id).keySet();
@@ -140,6 +168,36 @@ public class GlobalRealtimeCleanupImpl implements GlobalRealtimeCleanup, Managem
     public Collection<ID> removeFromResourceDirectory(Collection<ID> ids) throws OXException {
         return hazelcastResourceDirectory.remove(ids).keySet();
     }
+    
+    private void doCleanupForId(ID id) {
+        // Do the local cleanup via a simple service call
+        LocalRealtimeCleanup localRealtimeCleanup = Services.optService(LocalRealtimeCleanup.class);
+        if (localRealtimeCleanup == null) {
+            LOG.error(
+                "Unable to start local cleanup. Shutting down?",
+                RealtimeExceptionCodes.NEEDED_SERVICE_MISSING.create(LocalRealtimeCleanup.class));
+        } else {
+            localRealtimeCleanup.cleanForId(id);
+        }
+
+        // Remote cleanup via distributed MultiTask to remaining members of the cluster
+        HazelcastInstance hazelcastInstance;
+        try {
+            hazelcastInstance = HazelcastAccess.getHazelcastInstance();
+            Member localMember = HazelcastAccess.getLocalMember();
+            Set<Member> clusterMembers = new HashSet<Member>(hazelcastInstance.getCluster().getMembers());
+            if (!clusterMembers.remove(localMember)) {
+                LOG.warn("Couldn't remove local member from cluster members.");
+            }
+            if (!clusterMembers.isEmpty()) {
+                hazelcastInstance.getExecutorService("default").submitToMembers(new PortableCleanupDispatcher(id), clusterMembers);
+            } else {
+                LOG.debug("No other cluster members besides the local member. No further clean up necessary.");
+            }
+        } catch (Exception e) {
+            LOG.error("Failed to issue remote cleanup for {}.", id, e);
+        }
+    }
 
     @Override
     public ManagementObject<GlobalRealtimeCleanupMBean> getManagementObject() {
