@@ -50,36 +50,43 @@
 package com.openexchange.realtime.hazelcast.directory;
 
 import java.util.Collection;
+import java.util.Dictionary;
 import java.util.HashSet;
+import java.util.Iterator;
 import java.util.Map;
 import java.util.Map.Entry;
 import java.util.Set;
 import com.google.common.base.Optional;
 import com.hazelcast.core.EntryEvent;
-import com.hazelcast.core.EntryListener;
 import com.hazelcast.core.HazelcastInstance;
 import com.hazelcast.core.IMap;
 import com.hazelcast.core.Member;
 import com.hazelcast.core.MultiMap;
 import com.hazelcast.query.Predicate;
 import com.openexchange.exception.OXException;
-import com.openexchange.java.ConcurrentSet;
 import com.openexchange.management.ManagementAware;
 import com.openexchange.management.ManagementObject;
+import com.openexchange.osgi.ExceptionUtils;
+import com.openexchange.realtime.cleanup.AbstractRealtimeJanitor;
 import com.openexchange.realtime.cleanup.RealtimeJanitor;
+import com.openexchange.realtime.directory.DefaultResource;
 import com.openexchange.realtime.directory.DefaultResourceDirectory;
 import com.openexchange.realtime.directory.Resource;
+import com.openexchange.realtime.directory.RoutingInfo;
 import com.openexchange.realtime.hazelcast.channel.HazelcastAccess;
 import com.openexchange.realtime.hazelcast.management.HazelcastResourceDirectoryMBean;
 import com.openexchange.realtime.hazelcast.management.HazelcastResourceDirectoryManagement;
+import com.openexchange.realtime.hazelcast.serialization.directory.PortableMemberPredicate;
+import com.openexchange.realtime.hazelcast.serialization.directory.PortableResource;
+import com.openexchange.realtime.hazelcast.serialization.packet.PortableID;
 import com.openexchange.realtime.packet.ID;
 import com.openexchange.realtime.packet.Presence;
 import com.openexchange.realtime.util.IDMap;
 
 /**
- * {@link HazelcastResourceDirectory} - Keeps mappings of general {@link ID}s to full {@link ID}s and full {@link ID}s to {@link Resource}.
- * New DefaultResources that are added to this directory are automatically converted to HazelcastResources and extended with the local
- * Hazelcast Member as routing info.
+ * {@link HazelcastResourceDirectory} - Keeps mappings of general {@link ID}s to full {@link ID}s and full {@link ID}s to {@link Resource}s.
+ * New {@link ID}s and {@link DefaultResource}s that are added to this directory are automatically converted to {@link PortableID}s and
+ * {@link PortableResource}s and extended with {@link RoutingInfo}s based on the local Hazelcast {@link Member}.
  *
  * @author <a href="mailto:steffen.templin@open-xchange.com">Steffen Templin</a>
  * @author <a href="mailto:marc.arens@open-xchange.com">Marc Arens</a>
@@ -97,9 +104,6 @@ public class HazelcastResourceDirectory extends DefaultResourceDirectory impleme
 
     private final HazelcastResourceDirectoryManagement managementObject;
 
-    /** Keep track of synthetic ids */
-    private final Set<ID> syntheticIDs;
-
     /**
      * Initializes a new {@link HazelcastResourceDirectory}.
      *
@@ -112,42 +116,24 @@ public class HazelcastResourceDirectory extends DefaultResourceDirectory impleme
         this.id_map = id_map;
         this.resource_map = resource_map;
         this.managementObject = new HazelcastResourceDirectoryManagement(this);
-        syntheticIDs = new ConcurrentSet<ID>();
-        getResourceMapping().addEntryListener(new EntryListener<String, Map<String, Object>>() {
+        addResourceMappingEntryListener(new ResourceMappingEntryAdapter() {
 
             @Override
-            public void entryUpdated(EntryEvent<String, Map<String, Object>> event) { /* nothing */
-            }
-
-            @Override
-            public void entryRemoved(EntryEvent<String, Map<String, Object>> event) {
-                ID id = new ID(event.getKey());
-                boolean removed = syntheticIDs.remove(id);
-                if (removed) {
-                    LOG.debug("Removed id from refresh list: {}", id);
-                }
-            }
-
-            @Override
-            public void entryAdded(EntryEvent<String, Map<String, Object>> event) { /* nothing */
-            }
-
-            @Override
-            public void entryEvicted(EntryEvent<String, Map<String, Object>> event) {
-                String id = event.getKey();
+            public void entryEvicted(EntryEvent<PortableID, PortableResource> event) {
+                PortableID id = event.getKey();
                 Object source = event.getSource();
                 Member member = event.getMember();
                 try {
-                    if (getIDMapping().remove(new ID(id).toGeneralForm().toString(), id)) {
-                        LOG.debug("Source {} on Member: {} fired event. Removing mapping for '{}' due to eviction of according resource.", source, member, id);
+                    if (getIDMapping().remove(id.toGeneralForm(), id)) {
+                        LOG.debug(
+                            "Source {} on Member: {} fired event. Removing mapping for '{}' due to eviction of associated resource.",
+                            source,
+                            member,
+                            id);
                     }
                 } catch (OXException e) {
                     LOG.warn("Could not handle eviction for id '{}'", id, e);
                 }
-                boolean removed = syntheticIDs.remove(new ID(id));
-                if (removed) {
-                    LOG.debug("Removed id from refresh list: {}", id);
-                }
             }
         }, false);
     }
@@ -160,30 +146,32 @@ public class HazelcastResourceDirectory extends DefaultResourceDirectory impleme
     @Override
     public IDMap<Resource> get(ID id) throws OXException {
         IDMap<Resource> foundResources = new IDMap<Resource>();
-        if (id.isGeneralForm()) {
-            MultiMap<String, String> idMapping = getIDMapping();
-            Collection<String> concreteIds = idMapping.get(id.toString());
+        PortableID currentPortableId = new PortableID(id);
+
+        if (currentPortableId.isGeneralForm()) {
+            MultiMap<PortableID,PortableID> idMapping = getIDMapping();
+            Collection<PortableID> concreteIds = idMapping.get(currentPortableId);
             if (concreteIds != null && !concreteIds.isEmpty()) {
-                IMap<String, Map<String, Object>> allResources = getResourceMapping();
-                Map<String, Map<String, Object>> resources = allResources.getAll(new HashSet<String>(concreteIds));
+                IMap<PortableID,PortableResource> allResources = getResourceMapping();
+                Map<PortableID, PortableResource> resources = allResources.getAll(new HashSet<PortableID>(concreteIds));
                 if (resources != null) {
-                    for (Entry<String, Map<String, Object>> entry : resources.entrySet()) {
-                        ID foundId = new ID(entry.getKey());
-                        HazelcastResource foundResource = HazelcastResourceWrapper.unwrap(entry.getValue());
+                    for (Entry<PortableID, PortableResource> entry : resources.entrySet()) {
+                        PortableID foundId = entry.getKey();
+                        PortableResource foundResource = entry.getValue();
                         foundResources.put(foundId, foundResource);
                     }
                 }
             }
         } else {
-            IMap<String, Map<String, Object>> allResources = getResourceMapping();
-            Map<String, Object> resourceMap = allResources.get(id.toString());
-            HazelcastResource resource = HazelcastResourceWrapper.unwrap(resourceMap);
-            if (resource != null) {
-                foundResources.put(id, resource);
+            IMap<PortableID,PortableResource> allResources = getResourceMapping();
+            PortableResource portableResource = allResources.get(currentPortableId);
+            if (portableResource != null) {
+                foundResources.put(currentPortableId, portableResource);
             } else {
-                resource = conjureResource(id);
-                if (resource != null) {
-                    foundResources.put(id, resource);
+                // no matching resource for requested id, can we create one on demand?
+                portableResource = conjureResource(currentPortableId);
+                if (portableResource != null) {
+                    foundResources.put(currentPortableId, portableResource);
                 }
             }
         }
@@ -194,29 +182,29 @@ public class HazelcastResourceDirectory extends DefaultResourceDirectory impleme
     @Override
     public IDMap<Resource> get(Collection<ID> ids) throws OXException {
         IDMap<Resource> foundResources = new IDMap<Resource>();
-        Set<ID> generalIds = new HashSet<ID>();
-        Set<ID> resourceIds = new HashSet<ID>();
+        Set<PortableID> generalIds = new HashSet<PortableID>();
+        Set<PortableID> conreteIds = new HashSet<PortableID>();
         for (ID id : ids) {
             if (id.isGeneralForm()) {
-                generalIds.add(id);
+                generalIds.add(new PortableID(id));
             } else {
-                resourceIds.add(id);
+                conreteIds.add(new PortableID(id));
             }
         }
 
-        if (!resourceIds.isEmpty()) {
-            IMap<String, Map<String, Object>> allResources = getResourceMapping();
-            Map<String, Map<String, Object>> matchingResources = allResources.getAll(IDWrapper.idsToStringSet(resourceIds));
-            if (matchingResources != null) {
-                for (Entry<String, Map<String, Object>> entry : matchingResources.entrySet()) {
-                    ID foundID = new ID(entry.getKey());
-                    HazelcastResource foundResource = HazelcastResourceWrapper.unwrap(entry.getValue());
+        if (!conreteIds.isEmpty()) {
+            IMap<PortableID,PortableResource> allResources = getResourceMapping();
+            Map<PortableID, PortableResource> matchingPortableResources = allResources.getAll(conreteIds);
+            if (matchingPortableResources != null) {
+                for (Entry<PortableID, PortableResource> entry : matchingPortableResources.entrySet()) {
+                    ID foundID = entry.getKey();
+                    PortableResource foundResource = entry.getValue();
                     foundResources.put(foundID, foundResource);
                 }
                 // Remove all found Resources so we can try to conjure the rest
-                resourceIds.removeAll(foundResources.keySet());
-                for (ID id : resourceIds) {
-                    HazelcastResource resource = conjureResource(id);
+                conreteIds.removeAll(foundResources.keySet());
+                for (PortableID id : conreteIds) {
+                    PortableResource resource = conjureResource(id);
                     if (resource != null) {
                         foundResources.put(id, resource);
                     }
@@ -225,16 +213,16 @@ public class HazelcastResourceDirectory extends DefaultResourceDirectory impleme
         }
 
         if (!generalIds.isEmpty()) {
-            for (ID id : generalIds) {
-                MultiMap<String, String> idMapping = getIDMapping();
-                Collection<String> concreteIds = idMapping.get(id.toString());
-                if (concreteIds != null && !concreteIds.isEmpty()) {
-                    IMap<String, Map<String, Object>> allResources = getResourceMapping();
-                    Map<String, Map<String, Object>> resources = allResources.getAll(new HashSet<String>(concreteIds));
-                    if (resources != null) {
-                        for (Entry<String, Map<String, Object>> entry : resources.entrySet()) {
-                            ID foundID = new ID(entry.getKey());
-                            HazelcastResource foundResource = HazelcastResourceWrapper.unwrap(entry.getValue());
+            for (PortableID id : generalIds) {
+                MultiMap<PortableID,PortableID> idMapping = getIDMapping();
+                Collection<PortableID> foundConcreteIds = idMapping.get(id);
+                if (foundConcreteIds != null && !foundConcreteIds.isEmpty()) {
+                    IMap<PortableID,PortableResource> resourceMapping = getResourceMapping();
+                    Map<PortableID, PortableResource> foundResourcesForConcreteIDs = resourceMapping.getAll(new HashSet<PortableID>(foundConcreteIds));
+                    if (foundResourcesForConcreteIDs != null) {
+                        for (Entry<PortableID, PortableResource> entry : foundResourcesForConcreteIDs.entrySet()) {
+                            PortableID foundID = entry.getKey();
+                            PortableResource foundResource = entry.getValue();
                             foundResources.put(foundID, foundResource);
                         }
                     }
@@ -249,43 +237,42 @@ public class HazelcastResourceDirectory extends DefaultResourceDirectory impleme
     protected IDMap<Resource> doRemove(Collection<ID> ids) throws OXException {
         LOG.debug("Removing IDs from HazelcastResourceDirectory: {}", ids);
         IDMap<Resource> removedResources = new IDMap<Resource>();
-        Set<ID> generalIds = new HashSet<ID>();
-        Set<ID> resourceIds = new HashSet<ID>();
+        Set<PortableID> generalIds = new HashSet<PortableID>();
+        Set<PortableID> concreteIds = new HashSet<PortableID>();
         for (ID id : ids) {
             if (id.isGeneralForm()) {
-                generalIds.add(id);
+                generalIds.add(new PortableID(id));
             } else {
-                resourceIds.add(id);
+                concreteIds.add(new PortableID(id));
             }
         }
 
         try {
-            MultiMap<String, String> idMapping = getIDMapping();
-            IMap<String, Map<String, Object>> allResources = getResourceMapping();
-            if (!resourceIds.isEmpty()) {
-                for (ID id : resourceIds) {
-                    idMapping.remove(id.toGeneralForm().toString(), id.toString());
-                    Map<String, Object> resourceMap = allResources.remove(id.toString());
-                    if (resourceMap != null) {
-                        HazelcastResource removedResource = HazelcastResourceWrapper.unwrap(resourceMap);
+            MultiMap<PortableID,PortableID> idMapping = getIDMapping();
+            IMap<PortableID,PortableResource> allResources = getResourceMapping();
+            if (!concreteIds.isEmpty()) {
+                for (PortableID id : concreteIds) {
+                    idMapping.remove(id.toGeneralForm(), id);
+                    PortableResource removedResource = allResources.remove(id);
+                    if(removedResource != null) {
                         removedResources.put(id, removedResource);
                     }
                 }
             }
 
             if (!generalIds.isEmpty()) {
-                for (ID id : generalIds) {
-                    Collection<String> toRemove = idMapping.remove(id.toString());
-                    for (String concreteId : toRemove) {
-                        Map<String, Object> resourceMap = allResources.remove(concreteId);
-                        if (resourceMap != null) {
-                            HazelcastResource removedResource = HazelcastResourceWrapper.unwrap(resourceMap);
-                            removedResources.put(new ID(concreteId), removedResource);
+                for (PortableID generalId : generalIds) {
+                    Collection<PortableID> foundConcreteIds = idMapping.remove(generalId);
+                    for (PortableID concreteId : foundConcreteIds) {
+                        PortableResource removedResource = allResources.remove(concreteId);
+                        if(removedResource != null) {
+                            removedResources.put(concreteId, removedResource);
                         }
                     }
                 }
             }
         } catch (Throwable t) {
+            ExceptionUtils.handleThrowable(t);
             throw new OXException(t);
         }
         LOG.debug("Removed Resource(s) from HazelcastResourceDirectory: {}", removedResources);
@@ -295,30 +282,30 @@ public class HazelcastResourceDirectory extends DefaultResourceDirectory impleme
     @Override
     protected IDMap<Resource> doRemove(ID id) throws OXException {
         LOG.debug("Removing ID from HazelcastResourceDirectory: {}", id);
+        PortableID currentPortableId = new PortableID(id);
         IDMap<Resource> removedResources = new IDMap<Resource>();
         try {
-            MultiMap<String, String> idMapping = getIDMapping();
-            IMap<String, Map<String, Object>> allResources = getResourceMapping();
-            if (id.isGeneralForm()) {
-                Collection<String> toRemove = idMapping.remove(id.toString());
-                if (toRemove != null) {
-                    for (String concreteId : toRemove) {
-                        Map<String, Object> resourceMap = allResources.remove(concreteId);
-                        if (resourceMap != null) {
-                            HazelcastResource removedResource = HazelcastResourceWrapper.unwrap(resourceMap);
-                            removedResources.put(new ID(concreteId), removedResource);
+            MultiMap<PortableID,PortableID> idMapping = getIDMapping();
+            IMap<PortableID,PortableResource> allResources = getResourceMapping();
+            if (currentPortableId.isGeneralForm()) {
+                Collection<PortableID> removedConcreteIds = idMapping.remove(currentPortableId);
+                if (removedConcreteIds != null) {
+                    for (PortableID concreteId : removedConcreteIds) {
+                        PortableResource removedResource = allResources.remove(concreteId);
+                        if (removedResource != null) {
+                            removedResources.put(concreteId, removedResource);
                         }
                     }
                 }
             } else {
-                idMapping.remove(id.toGeneralForm().toString(), id.toString());
-                Map<String, Object> resourceMap = allResources.remove(id.toString());
-                if (resourceMap != null) {
-                    HazelcastResource removedResource = HazelcastResourceWrapper.unwrap(resourceMap);
-                    removedResources.put(id, removedResource);
+                idMapping.remove(currentPortableId.toGeneralForm(), currentPortableId);
+                PortableResource removedResource = allResources.remove(currentPortableId);
+                if (removedResource != null) {
+                    removedResources.put(currentPortableId, removedResource);
                 }
             }
         } catch (Throwable t) {
+            ExceptionUtils.handleThrowable(t);
             throw new OXException(t);
         }
         LOG.debug("Removed Resource(s) from HazelcastResourceDirectory: {}", removedResources);
@@ -326,50 +313,49 @@ public class HazelcastResourceDirectory extends DefaultResourceDirectory impleme
     }
 
     /*
-     * Automatically converts incoming Resources to HazelcastResources (which extends them with hazelcast specific routing infos) before
-     * adding them to the hazelcast data structures.
+     * Automatically converts incoming Resources to HazelcastResources (which extends them with hazelcast specific routing infos) and
+     * infcoming IDs to PortableIDs before adding them to the hazelcast data structures.
      */
     @Override
-    protected HazelcastResource doSet(ID id, Resource resource, boolean overwrite) throws OXException {
-        HazelcastResource hazelcastResource = new HazelcastResource(resource);
+    protected PortableResource doSet(ID id, Resource resource, boolean overwrite) throws OXException {
+        PortableResource currentPortableResource = new PortableResource(resource, HazelcastAccess.getLocalMember());
+        PortableID currentPortableID = new PortableID(id);
 
-        HazelcastResource previousResource = null;
+        PortableResource previousPortableResource = null;
         try {
-            MultiMap<String, String> idMapping = getIDMapping();
-            IMap<String, Map<String, Object>> allResources = getResourceMapping();
-            idMapping.put(id.toGeneralForm().toString(), id.toString());
+            MultiMap<PortableID,PortableID> idMapping = getIDMapping();
+            IMap<PortableID,PortableResource> resourceMapping = getResourceMapping();
+            idMapping.put(currentPortableID.toGeneralForm(), currentPortableID);
 
             // don't overwrite exisiting Presence Data
-            if (hazelcastResource.getPresence() == null) { // a DefaultResource / idle reconnect
-                Map<String, Object> previousResourceMap = allResources.get(id.toString());
-                if (previousResourceMap != null) {
-                    previousResource = HazelcastResourceWrapper.unwrap(previousResourceMap);
-                }
-                if (previousResource != null && previousResource.getPresence() != null) {
-                    hazelcastResource.setPresence(previousResource.getPresence());
-                    allResources.put(id.toString(), HazelcastResourceWrapper.wrap(hazelcastResource));
+            if (currentPortableResource.getPresence() == null) {
+                // current resource doesn't provide presence infos, might be a DefaultResource / idle reconnect
+                previousPortableResource = resourceMapping.get(currentPortableID);
+                if(previousPortableResource != null && previousPortableResource.getPresence() != null) {
+                    // but the previous resource provides a presence
+                    currentPortableResource.setPresence(previousPortableResource.getPresence());
+                    resourceMapping.put(currentPortableID, currentPortableResource);
                 } else {
-                    if (overwrite) {
-                        previousResourceMap = allResources.put(id.toString(), HazelcastResourceWrapper.wrap(hazelcastResource));
-                        previousResource = HazelcastResourceWrapper.unwrap(previousResourceMap);
+                    // neither current nor previous resource provide presence infos
+                    if(overwrite) {
+                        previousPortableResource = resourceMapping.put(currentPortableID, currentPortableResource);
                     } else {
-                        previousResourceMap = allResources.putIfAbsent(id.toString(), HazelcastResourceWrapper.wrap(hazelcastResource));
-                        previousResource = HazelcastResourceWrapper.unwrap(previousResourceMap);
+                        previousPortableResource = resourceMapping.putIfAbsent(currentPortableID, currentPortableResource);
                     }
                 }
-            } else { // a Resource with Presence data
-                Map<String, Object> previousResourceMap = null;
-                if (overwrite) {
-                    previousResourceMap = allResources.put(id.toString(), HazelcastResourceWrapper.wrap(hazelcastResource));
+            } else {
+                // current resource provides Presence data
+                if(overwrite) {
+                    previousPortableResource = resourceMapping.put(currentPortableID, currentPortableResource);
                 } else {
-                    previousResourceMap = allResources.putIfAbsent(id.toString(), HazelcastResourceWrapper.wrap(hazelcastResource));
+                    previousPortableResource = resourceMapping.putIfAbsent(currentPortableID, currentPortableResource);
                 }
-                previousResource = HazelcastResourceWrapper.unwrap(previousResourceMap);
             }
         } catch (Throwable t) {
+            ExceptionUtils.handleThrowable(t);
             throw new OXException(t);
         }
-        return previousResource;
+        return previousPortableResource;
     }
 
     @Override
@@ -391,19 +377,20 @@ public class HazelcastResourceDirectory extends DefaultResourceDirectory impleme
     }
 
     /**
-     * Try to create a new HazelcastResource for the given ID. One place where this is used is during creation of GroupDispatchers.
+     * Try to create a new Resource for the given ID and persist it in the ResourceDirectory. One place where this is used is during
+     * creation of GroupDispatchers.
      *
      * @param id The ID used to reach a Resource
      * @return null if the HazelcastResource couldn't be created, otherwise the new Resource
      * @throws OXException
      */
-    private HazelcastResource conjureResource(ID id) throws OXException {
+    private PortableResource conjureResource(PortableID id) throws OXException {
         if (!conjure(id)) {
             return null;
         }
-        syntheticIDs.add(id);
-        HazelcastResource res = new HazelcastResource();
-        HazelcastResource meantime = setIfAbsent(id, res);
+
+        PortableResource res = new PortableResource(HazelcastAccess.getLocalMember());
+        PortableResource meantime = setIfAbsent(id, res);
         if (meantime == null) {
             return res;
         }
@@ -411,11 +398,11 @@ public class HazelcastResourceDirectory extends DefaultResourceDirectory impleme
     }
 
     /*
-     * Hazelcast specific setIfAbsent that returns a proper HazelcastResource
+     * Hazelcast specific setIfAbsent that returns a proper PortableResource
      */
     @Override
-    public HazelcastResource setIfAbsent(ID id, Resource resource) throws OXException {
-        HazelcastResource previousResource = doSet(id, resource, false);
+    public PortableResource setIfAbsent(ID id, Resource resource) throws OXException {
+        PortableResource previousResource = doSet(id, resource, false);
         if (null == previousResource) {
             notifyAdded(id, resource);
         }
@@ -428,18 +415,19 @@ public class HazelcastResourceDirectory extends DefaultResourceDirectory impleme
      * @return the map used for mapping general IDs to full IDs.
      * @throws OXException if the HazelcastInstance is missing.
      */
-    public MultiMap<String, String> getIDMapping() throws OXException {
+    public MultiMap<PortableID, PortableID> getIDMapping() throws OXException {
         HazelcastInstance hazelcast = HazelcastAccess.getHazelcastInstance();
         return hazelcast.getMultiMap(id_map);
     }
 
     /**
-     * Get the mapping of full IDs to the Resource e.g. ox://marc.arens@premium/random <-> ResourceMap.
+     * Get the mapping of full IDs to the Resource e.g. ox://marc.arens@premium/random <-> Resource. The resource includes the
+     * {@link RoutingInfo} needed to address clients identified by the {@link ID}
      *
      * @return the map used for mapping full IDs to ResourceMaps.
      * @throws OXException if the map couldn't be fetched from hazelcast
      */
-    public IMap<String, Map<String, Object>> getResourceMapping() throws OXException {
+    public IMap<PortableID, PortableResource> getResourceMapping() throws OXException {
         HazelcastInstance hazelcast = HazelcastAccess.getHazelcastInstance();
         return hazelcast.getMap(resource_map);
     }
@@ -456,14 +444,14 @@ public class HazelcastResourceDirectory extends DefaultResourceDirectory impleme
     }
 
     /**
-     * Add a {@link ResourceMappingEntryListener} to this ResourceDirectory. 
+     * Add a {@link ResourceMappingEntryListener} to this ResourceDirectory.
      * @param entryListener the {@link ResourceMappingEntryListener} to add
      * @param includeValue should the value be included when informing the listener
      * @return the registration id that can be used to remove a previously added listener
      * @throws OXException
      */
     public String addResourceMappingEntryListener(ResourceMappingEntryListener entryListener, boolean includeValue) throws OXException {
-        Optional<Predicate<String, Map<String, Object>>> predicate = entryListener.getPredicate();
+        Optional<Predicate<PortableID, PortableResource>> predicate = entryListener.getPredicate();
         if(predicate.isPresent()) {
             return getResourceMapping().addEntryListener(entryListener, predicate.get(), includeValue);
         } else {
@@ -481,4 +469,34 @@ public class HazelcastResourceDirectory extends DefaultResourceDirectory impleme
         return getResourceMapping().removeEntryListener(registrationId);
     }
 
+    /**
+     * Find all Resources in this directory that are located on a given member node.
+     *
+     * @param member The cluster member
+     * @return all Resources in this directory that are located on the given member node.
+     * @throws OXException
+     */
+    public IDMap<Resource> getResourcesOfMember(Member member)throws OXException {
+        IMap<PortableID, PortableResource> allResources = getResourceMapping();
+        PortableMemberPredicate memberPredicate = new PortableMemberPredicate(member);
+        Set<Entry<PortableID, PortableResource>> matchingResources = allResources.entrySet(memberPredicate);
+
+        IDMap<Resource> foundIds = new IDMap<Resource>();
+        Iterator<Entry<PortableID, PortableResource>> iterator = matchingResources.iterator();
+        while(iterator.hasNext()) {
+            try {
+                Entry<PortableID, PortableResource> next = iterator.next();
+                foundIds.put(next.getKey(), next.getValue());
+            } catch (Exception e) {
+                LOG.error("Couldn't add resource that was found for member {}", member, e);
+            }
+        }
+        return foundIds;
+    }
+
+    @Override
+    public Dictionary<String, Object> getServiceProperties() {
+        return AbstractRealtimeJanitor.NO_PROPERTIES;
+    }
+
 }
