@@ -50,6 +50,7 @@
 package com.openexchange.realtime.packet;
 
 import com.openexchange.exception.OXException;
+import com.openexchange.realtime.payload.PayloadTree;
 
 /**
  * {@link IQ} - Used for command exchanges.
@@ -129,4 +130,9 @@ public class IQ extends Stanza {
         return new IQ();
     }
 
+    @Override
+    public void addPayload(PayloadTree tree) {
+        addPayloadToMap(tree, payloads);
+    }
+
 }
