@@ -148,6 +148,9 @@ public class Presence extends Stanza {
         defaultElements.add(STATUS_PATH);
         defaultElements.add(PRIORITY_PATH);
         defaultElements.add(ERROR_PATH);
+        setMessage(message);
+        setState(state);
+        setPriority(priority);
     }
 
     /**
@@ -381,6 +384,15 @@ public class Presence extends Stanza {
         }
     }
 
+    @Override
+    public void addPayload(PayloadTree tree) {
+        ElementPath newRootPath = tree.getElementPath();
+        if (defaultElements.contains(newRootPath)) {
+            payloads.remove(newRootPath);
+        }
+        addPayloadToMap(tree, payloads);
+    }
+
     /**
      * Initialize a Presence Builder
      * 
